import 'dart:async';

import 'package:flutter/widgets.dart';

import '../viewmodel/observable_list.dart';
import '../viewmodel/inotify_property_changed.dart';

class AnimatedObservableList<TItem> extends StatefulWidget {
  // Properties

  final ObservableList<TItem> list;
  final AnimatedListItemBuilder itemBuilder;
  final AnimatedListRemovedItemBuilder removeItemBuilder;

  // Methods
  AnimatedObservableList(
      {Key key,
      @required this.list,
      @required this.itemBuilder,
      @required this.removeItemBuilder})
      : assert(list != null),
        assert(itemBuilder != null),
        assert(removeItemBuilder != null),
        super(key: key);

  @override
  _AnimatedObservableListState<TItem> createState() =>
      _AnimatedObservableListState<TItem>(list);
}

class _AnimatedObservableListState<TItem>
    extends State<AnimatedObservableList<TItem>> {
  // Properties
  StreamSubscription<PropertyChangedEvent> _subscription;
  final ObjectKey _animatedListKey;

  _AnimatedObservableListState(ObservableList<TItem> list)
      : _animatedListKey = ObjectKey(list);

  // Methods
  @override
  void initState() {
    super.initState();
    _subscribe();
  }

  @override
  void dispose() {
    _unsubscribe();
    super.dispose();
  }

  void _subscribe() {
    if (widget.list != null) {
      _subscription = widget.list
          .whenAnyOfPropertiesChanged(Set<String>.of(<String>[
        ItemAddedEvent.operationName,
        ItemRemovedEvent.operationName
      ]))
          .listen(
        (PropertyChangedEvent data) {
          final animatedListState =
              (_animatedListKey.value as AnimatedListState);

          if (data is ItemAddedEvent) {
            animatedListState.insertItem(data.index);
          }

          if (data is ItemRemovedEvent) {
            animatedListState.removeItem(data.index, widget.removeItemBuilder);
          }
        },
        onError: (Object error) {},
        onDone: () {},
      );
    }
  }

  void _unsubscribe() {
    if (_subscription != null) {
      _subscription.cancel();
      _subscription = null;
    }
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedList(
      key: _animatedListKey,
      itemBuilder: widget.itemBuilder,
      initialItemCount: widget.list.length,
    );
  }
}
